% this code is used to generate plot 'figexper3_Greedy.eps' for the toy example in section 4.3.
% this code shows results for paired t-test between realized cost under LQR and Greedy policy 
%
% A, B is the environment dynamic;
% Ahat is forecast matrix
% x0 is the scalar initial state
% T is the time horizon
% bar is the attack reference target (value)
% lambdatilde is \tilde\lambda in paper
% sim is the number of trials

clear;

alpha = [0,-[-0.4    0.3    0.7]];
alphahat = [0, -[-0.41    0.29    0.68]];

p = 3;

B = [0;1;0;0];

A = diag(ones(1,p),-1);
Ahat = diag(ones(1,p),-1);
A(1,1) = 1;
Ahat(1,1) = 1;
A(2,:) = alpha;
Ahat(2,:) = alphahat;


x0 = 10;
T = 15;

sim = 50;

balance = 10^(-1);

% set lambda to balance the different sum number
lambda = @(beta) balance*sum(sum(beta))/T;

F = @(x, u, w, mode) A * x + B * u + B * w;

% when attacak target is to change tomorrow's prediction
beta_tom = diag(ones(1,T),1);
[xhat, x, u] = Pred(F, Ahat, x0, T, zeros(1, T), zeros(1, T));
xbar_tom = diag(0.5*x(2,2:T+1),1);

% initialize cost
cost_lqr_tom = zeros(1, sim);
cost_G_tom = zeros(1, sim);
cost_non_tom = zeros(1, sim);


% used fixed random seed to generate noise sequence
rng(1);

for iSim = 1:sim
	w = normrnd(0, 0.1, [1, T]);	
	
	% when there is no attack, compute prediction, state value, control sequence (zeros)
	[xhat_non, x_non, u_non] = Pred(F, Ahat, x0, T, zeros(1, T), w);		

	% compute policy functions of LQR anf Greedy
	PHI_lqr_tom = LQRStateAttack(A,Ahat,xbar_tom,beta_tom,lambda(beta_tom),T);
	PHI_G_tom = GreedyLinear(A,Ahat,xbar_tom,beta_tom,lambda(beta_tom),T);	

	% compute state, attack sequence and prediction sequence uder LQR and Greedy policy
	[xhat_lqr_tom, x_lqr_tom, u_lqr_tom] = PredAttPlo(F, Ahat, x0, T, @(t,x) PHI_lqr_tom(t,x), w);
	[xhat_G_tom, x_G_tom, u_G_tom] = PredAttPlo(F, Ahat, x0, T, @(t,x) PHI_G_tom(t,x), w);	

	cost_lqr_tom(iSim) = Cost(Ahat, xbar_tom, beta_tom, x_lqr_tom, u_lqr_tom, lambda(beta_tom), T);
	cost_G_tom(iSim) = Cost(Ahat, xbar_tom, beta_tom, x_G_tom, u_G_tom, lambda(beta_tom), T);
	cost_non_tom(iSim) = Cost(Ahat, xbar_tom, beta_tom, x_non, u_non, lambda(beta_tom), T);	
			
end

figtoy2 = figure

% no attack
ax1 = subplot(1,3,1);
for t = 0 : T-1
	p1 = plot([t+1:T], xhat_non(t+1, t+2:T+1), '-b.'); hold on;
end
p2 = plot([0:T], x_non(2,:), '-m.');
xlabel('time');
title(['no attack']);
axis('square');
legend([p1, p2], {'forecast $\hat x_{t'' \mid t} $ ', 'env state $x_t$'}, 'Location', 'northwest','interpreter','latex');

% use LQR policy
ax2 = subplot(1,3,2);
[p1, p2, p3] = Predplot(xhat_lqr_tom, x_lqr_tom, u_lqr_tom, T);
p4 = plot([1:T], diag(xbar_tom, 1), '-g.');
p5 = plot([1:T], diag(xhat_lqr_tom, 1), '*b');
legend([p3, p4], {'LQR $u_t$', 'target $\bar x_{t+1 \mid t}$'}, 'Location', 'northwest','interpreter','latex');
str0 = ['attack (LQR): $\hat x_{t+1 \mid t}$'];
str1 = ['LQR cost = ', num2str(cost_lqr_tom(sim))];
str2 = ['Greedy cost = ', num2str(cost_G_tom(sim))]; 
str3 = ['no attack cost = ', num2str(cost_non_tom(sim))];
str = {str0};
title(str,'interpreter','latex');
axis('square');

linkaxes([ax1,ax2],'xy')

% use Greedy policy
ax3 = subplot(1,3,3);
for t = 0 : T-1
	p1 = plot([t+1:T], xhat_G_tom(t+1, t+2:T+1), '-b.'); hold on;
end
p2 = plot([0:T], x_G_tom(2,:), '-m.');
p3 = plot([0:T-1], u_G_tom, ':r.');
ref1 = refline(0,10);
ref2 = refline(0,-10);
ref1.Color = 'k';
ref2.Color = 'k';
xlabel('time');
p4 = plot([1:T], diag(xbar_tom, 1), '-g.');
p5 = plot([1:T], diag(xhat_G_tom, 1), '*b');
legend([p3, p4], { 'Greedy $u_t$', 'target $\bar x_{t+1 \mid t}$'}, 'Location', 'northwest','interpreter','latex');
str0 = ['attack (Greedy): $\hat x_{t+1 \mid t}$'];
str1 = ['LQR cost = ', num2str(cost_lqr_tom(sim))];
str2 = ['Greedy cost = ', num2str(cost_G_tom(sim))]; 
str3 = ['no attack cost = ', num2str(cost_non_tom(sim))];
str = {str0};
title(str,'interpreter','latex');
axis('square');
% ----------------------------------------------------------------------------------


saveas(figtoy2,'figexper3_Greedy','epsc');

% report mean cost and run paired t-test for 50 pairs of realized cost under LQR and Greedy policy
disp({str0;str1;str2;str3});
disp(' ');
disp('average cost and t-test when:');
disp(' ');
disp('attack tomorrow''s prediction:');
disp({'no attack cost: = ', mean(cost_non_tom), std(cost_non_tom)/sqrt(length(cost_non_tom))});
disp({'lqr cost: = ', mean(cost_lqr_tom),std(cost_lqr_tom)/sqrt(length(cost_lqr_tom))});
disp({'Greedy cost: = ', mean(cost_G_tom),std(cost_G_tom)/sqrt(length(cost_G_tom))});
disp('paired t-test, H0: cost_lqr = cost_G; H1: cost_lqr ~= cost_G;');
[h,p] = ttest(cost_lqr_tom,cost_G_tom);
disp({'p-value = ', p});


function [p1, p2, p3] = Predplot(xhat, x, u, T)
for t = 0 : T-1
	p1 = plot([t+1:T], xhat(t+1, t+2:T+1), '-b.'); hold on;
end
p2 = plot([0:T], x(2,:), '-m.');
p3 = plot([0:T-1], u, '-r.');

xlabel('time');

end